#include <Servo.h>
#include <LiquidCrystal_I2C.h>

enum Pins {
    EchoPin = 2,      // Echo pin number
    TrigPin = 3,      // Trigger pin number
    BuzzerPin = 7,    // Buzzer pin number
    ServoPin = 6      // Servo pin number
};

const int MaxScanDistance = 70;  // Maximum distance for scan

// LCD and Servo objects creation
LiquidCrystal_I2C lcd(0x27,16,2);
Servo servo;

// Variables for pulse duration and distance
float pulseDuration;
float detectedDistance;

// Set up the required pins
void configurePins() {
    pinMode(Pins::TrigPin, OUTPUT);
    pinMode(Pins::EchoPin, INPUT);
    pinMode(Pins::BuzzerPin, OUTPUT);
}

// Attach servo to its pin
void configureServo() {
    servo.attach(Pins::ServoPin);
}

// Initialize and light LCD
void configureLCD() {
    lcd.init();
    lcd.backlight();
}

// Set up environment for operation
void setup() {
    configurePins();
    configureServo();
    configureLCD();
    Serial.begin(9600);
}

// Main loop, scanning for intruders
void loop() {
    for (int angle = 0; angle <= 180; angle += 2) {
        servo.write(angle);
        measureTime();
        detectedDistance = pulseDuration * 0.01715;
        handleDetection(detectedDistance);
        delay(100);
    }
}

// Measure time for echo pulse
void measureTime() {
    digitalWrite(Pins::TrigPin, LOW);
    delayMicroseconds(2);
    digitalWrite(Pins::TrigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(Pins::TrigPin, LOW);
    pulseDuration = pulseIn(Pins::EchoPin, HIGH);
}

// Handles detection distance processing
void handleDetection(float distance) {
    if (distance <= MaxScanDistance) {
        alertIntruder(distance);
    } else {
        lcd.clear();
        lcd.print("Start searching");
    }
}

// Alert for detected object
void alertIntruder(float distance) {
    lcd.setBacklight(1);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Movement seen at: ");
    lcd.setCursor(0, 1);
    lcd.print("    ");
    lcd.print(distance);
    lcd.print(" cm");
    soundBuzzer();
}

// Emit sound alert for detection
void soundBuzzer() {
    digitalWrite(Pins::BuzzerPin, HIGH);
    delay(3000);
    digitalWrite(Pins::BuzzerPin, LOW);
}